@@ -1,11 +1,14 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016 Sqreen. All Rights Reserved.
-# Please refer to our terms for more information: https://www.sqreen.io/terms.html
+# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Please refer to our terms for more information:
+#
+#     https://www.sqreen.io/terms.html
+#
 """ Simple delivery method that directly call session on event
 """
 
+from ..events import Attack, RequestRecord
 from ..remote_exception import RemoteException
-from ..events import Attack
 
 
 class SimpleDeliverer(object):
@@ -13,6 +16,10 @@ class SimpleDeliverer(object):
     on their types
     """
 
+    batch_size = 0
+    original_max_staleness = 0
+    max_staleness = 0
+
     def __init__(self, session):
         self.session = session
 
@@ -23,6 +30,8 @@ class SimpleDeliverer(object):
             return self.session.post_sqreen_exception(event.to_dict())
         if isinstance(event, Attack):
             return self.session.post_attack(event.to_dict())
+        if isinstance(event, RequestRecord):
+            return self.session.post_request_record(event.to_dict())
         else:
             err_msg = "Unknown event type {}".format(type(event))
             raise NotImplementedError(err_msg)
