@@ -1,20 +1,62 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016 Sqreen. All Rights Reserved.
-# Please refer to our terms for more information: https://www.sqreen.io/terms.html
+# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Please refer to our terms for more information:
+#
+#     https://www.sqreen.io/terms.html
+#
 """ DBApi2 strategy and custom connection and cursor classes
 """
-import sys
-import logging
 import inspect
+import logging
 
-from ..helpers import partial, Proxy
+from ...utils import update_wrapper
+from ..helpers import Proxy, partial
 from ..hook_point import hook_point
-from ..import_hook import ImportHook
-
 from .base import BaseStrategy
 
-
 LOGGER = logging.getLogger(__name__)
+METHODS_CACHE = {}
+
+
+def get_methods_and_builtin(object):
+    """ Iterate on an object and returns its attribute that are either
+    methods or builtins
+    """
+    object_class = object.__class__
+    result = {}
+
+    # Check cache first
+    cache_result = METHODS_CACHE.get(object_class, None)
+
+    # If not cached do the whole check
+    if cache_result is None:
+        for key in dir(object.__class__):
+            # Ignore private or semi-private methods
+            if key.startswith('_'):
+                continue
+
+            try:
+                value = getattr(object, key)
+            except AttributeError:
+                continue
+
+            if inspect.ismethod(value) or inspect.isbuiltin(value):
+                result[key] = value
+
+        # Cache the computation
+        METHODS_CACHE[object_class] = result.keys()
+    # If cached, retrieve only the methods, don't check against that it's a
+    # method or a builtin
+    else:
+        for key in cache_result:
+            try:
+                value = getattr(object, key)
+            except AttributeError:
+                continue
+
+            result[key] = value
+
+    return result
 
 
 class CustomCursor(Proxy):
@@ -24,19 +66,14 @@ class CustomCursor(Proxy):
 
     def __init__(self, real_cursor, hook_point, module_path):
         super(CustomCursor, self).__init__(real_cursor)
+        hook_module = "{}::Cursor".format(module_path)
 
-        methods = inspect.getmembers(self._obj, inspect.ismethod)
-        builtins = inspect.getmembers(self._obj, inspect.isbuiltin)
+        members = get_methods_and_builtin(self._obj)
 
-        for method in methods + builtins:
-            if method[0].startswith('_'):
-                # Ignore private or semi-private methods
-                continue
-
-            hook_module = "{}::Cursor".format(module_path)
-            new_hook_point = partial(hook_point, hook_module, method[0],
-                                     method[1])
-            object.__setattr__(self, method[0], new_hook_point)
+        for key, method in members.items():
+            new_hook_point = hook_point(hook_module, key,
+                                        method)
+            object.__setattr__(self, key, new_hook_point)
 
 
 class CustomConnection(Proxy):
@@ -49,23 +86,17 @@ class CustomConnection(Proxy):
         self._sqreen_hook_point = hook_point
         self._sqreen_module_path = module_path
         super(CustomConnection, self).__init__(real_connection)
+        hook_module = "{}::Connection".format(module_path)
 
-        methods = inspect.getmembers(self._obj, inspect.ismethod)
-        builtins = inspect.getmembers(self._obj, inspect.isbuiltin)
+        members = get_methods_and_builtin(self._obj)
 
-        for method in methods + builtins:
-            if method[0] == 'cursor':
+        for key, method in members.items():
+            if key == 'cursor':
                 # Don't hook on cursor
                 continue
 
-            if method[0].startswith('_'):
-                # Ignore private or semi-private methods
-                continue
-
-            hook_module = "{}::Connection".format(module_path)
-            new_hook_point = partial(hook_point, hook_module, method[0],
-                                     method[1])
-            object.__setattr__(self, method[0], new_hook_point)
+            new_hook_point = hook_point(hook_module, key, method)
+            object.__setattr__(self, key, new_hook_point)
 
     def cursor(self, *args, **kwargs):
         """ Instantiate a real cursor, proxy it via CustomCursor.
@@ -79,7 +110,12 @@ def custom_connect(hook_point, module_path, original_connect, *args, **kwargs):
     instantiate a connection via the original connect function and proxy it
     via CustomConnection defined earlier.
     """
-    return CustomConnection(original_connect(*args, **kwargs), hook_point, module_path)
+
+    def wrapper(*args, **kwargs):
+        return CustomConnection(original_connect(*args, **kwargs), hook_point, module_path)
+
+    wrapper = update_wrapper(wrapper, original_connect)
+    return wrapper
 
 
 class DBApi2Strategy(BaseStrategy):
@@ -106,13 +142,13 @@ class DBApi2Strategy(BaseStrategy):
     'psycopg2::Cursor' will correctly reference all psycopg2 cursor methods.
     """
 
-    def __init__(self, module_path, channel, before_hook_point=None):
-        super(DBApi2Strategy, self).__init__(channel, before_hook_point)
+    def __init__(self, module_path, observation_queue, queue, import_hook, before_hook_point=None):
+        super(DBApi2Strategy, self).__init__(observation_queue, queue, import_hook, before_hook_point)
         self.module_path = module_path
 
         self.hooked = False
 
-    def hook(self, callback):
+    def hook(self):
         """ Accept a callback and store it. If it's the first callback
         for this strategy, actually hook to the endpoint.
 
@@ -120,28 +156,18 @@ class DBApi2Strategy(BaseStrategy):
         But if no callback is defined on a specific method, the overhead will
         be minimal.
         """
-        # Register callback
-        self.add_callback(callback)
-
         # Check if we already hooked at
         if not self.hooked:
-
-            import_hook = ImportHook(self.module_path, self.import_hook_callback)
-            sys.meta_path.insert(0, import_hook)
+            self.import_hook.register_patcher(self.module_path, None, 'connect', self.import_hook_callback)
 
             self.hooked = True
 
-    def import_hook_callback(self, module):
+    def import_hook_callback(self, original):
         """ Monkey-patch the object located at hook_class.hook_name on an
         already loaded module
         """
-        original = getattr(module, "connect", None)
         _hook_point = partial(hook_point, self)
-        new_connect = partial(custom_connect, _hook_point,
-                              self.module_path, original)
-        setattr(module, "connect", new_connect)
-        LOGGER.debug("Successfully hooking on %s %s", self.module_path,
-                     "connect")
+        return custom_connect(_hook_point, self.module_path, original)
 
     @staticmethod
     def get_strategy_id(callback):
