@@ -1,33 +1,29 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016 Sqreen. All Rights Reserved.
-# Please refer to our terms for more information: https://www.sqreen.io/terms.html
+# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Please refer to our terms for more information:
+#
+#     https://www.sqreen.io/terms.html
+#
 """ Aggregate authentication tentatives
 """
-import logging
 import json
+import logging
 
+from ..runtime_storage import runtime
+from ..utils import CustomJSONEncoder
 from .matcher_callback import MatcherRule
-from ..runtime_infos import runtime
 
 LOGGER = logging.getLogger(__name__)
 
 
-class CustomEncoder(json.JSONEncoder):
-
-    def default(self, obj):
-        """ Return the repr of unkown objects
-        """
-        return repr(obj)
-
-
 class AuthMetricsCB(MatcherRule):
 
-    def post(self, result, original, *args, **kwargs):
+    def post(self, original, result, *args, **kwargs):
         # Django authentication model return either an User or None
         if result is None:
-            auth_status = 'login-fail'
+            auth_status = 'auto-login-fail'
         else:
-            auth_status = 'login-success'
+            auth_status = 'auto-login-success'
 
         keys = []
 
@@ -41,7 +37,11 @@ class AuthMetricsCB(MatcherRule):
             return
 
         request = runtime.get_current_request()
+        if not request:
+            LOGGER.warning("No request was recorded, abort")
+            return
         key = {'keys': keys, 'ip': request.client_ip}
 
-        observation_key = json.dumps(key, separators=(',', ':'), sort_keys=True, cls=CustomEncoder)
+        observation_key = json.dumps(key, separators=(',', ':'), sort_keys=True,
+                                     cls=CustomJSONEncoder)
         self.record_observation(auth_status, observation_key, 1)
