@@ -1,6 +1,9 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016 Sqreen. All Rights Reserved.
-# Please refer to our terms for more information: https://www.sqreen.io/terms.html
+# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Please refer to our terms for more information:
+#
+#     https://www.sqreen.io/terms.html
+#
 """ Count http codes
 """
 import logging
@@ -29,7 +32,7 @@ def wraps_start_response(original_start_response, callback):
                 if status_code:
                     callback.record_observation('http_code', status_code, 1)
         except Exception:
-            LOGGER.exception("Exception in {}".format(callable.__class__.__name__))
+            LOGGER.exception("Exception in %s", callable.__class__.__name__)
 
         return original_start_response(status, response_headers, exc_info)
 
