@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016 Sqreen. All Rights Reserved.
-# Please refer to our terms for more information: https://www.sqreen.io/terms.html
+# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Please refer to our terms for more information:
+#
+#     https://www.sqreen.io/terms.html
+#
 """ Look for known crawlers user-agents
 """
 
 from logging import getLogger
 
+from ..runtime_storage import runtime
 from .matcher_callback import MatcherRule
-from ..runtime_infos import runtime
-
 
 LOGGER = getLogger(__name__)
 
