@@ -1,19 +1,26 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016 Sqreen. All Rights Reserved.
-# Please refer to our terms for more information: https://www.sqreen.io/terms.html
+# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Please refer to our terms for more information:
+#
+#     https://www.sqreen.io/terms.html
+#
 """ Base class for matcher based rules
 """
-from logging import getLogger
 from collections import Mapping
+from logging import getLogger
 
-from ..rules import RuleCallback
-from ..matcher import Matcher
 from ..exceptions import InvalidArgument
+from ..matcher import Matcher
+from ..rules import RuleCallback
 
 LOGGER = getLogger(__name__)
 
 
 class MatcherRule(RuleCallback):
+    """ Base class for matcher based rules.
+    The valid matcher syntax is described here:
+    https://github.com/sqreen/Wiki/wiki/String-matcher
+    """
 
     def __init__(self, *args, **kwargs):
         super(MatcherRule, self).__init__(*args, **kwargs)
