@@ -1,12 +1,15 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016 Sqreen. All Rights Reserved.
-# Please refer to our terms for more information: https://www.sqreen.io/terms.html
+# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Please refer to our terms for more information:
+#
+#     https://www.sqreen.io/terms.html
+#
 """ Look for 404 error
 """
 import logging
 
 from ..rules import RuleCallback
-from ..runtime_infos import runtime
+from ..runtime_storage import runtime
 
 LOGGER = logging.getLogger(__name__)
 
