@@ -1,21 +1,23 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016 Sqreen. All Rights Reserved.
-# Please refer to our terms for more information: https://www.sqreen.io/terms.html
+# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Please refer to our terms for more information:
+#
+#     https://www.sqreen.io/terms.html
+#
 """ Look for badly behaved clients
 """
 from logging import getLogger
 
+from ..frameworks.django_framework import DjangoRequest
 from ..rules import RuleCallback
-from ..runtime_infos import runtime
-from ..frameworks.django import DjangoRequest
-
+from ..runtime_storage import runtime
 
 LOGGER = getLogger(__name__)
 
 
 class NotFoundCBDjango(RuleCallback):
 
-    def post(self, response, original, request):
+    def post(self, original, response, request):
         if response.status_code == 404:
             current_request = runtime.get_current_request()
 
